;-----------------------------------------------------------------------------
; READ D64 - D64 EXTRACTOR
; This utility extracts files from .d64 file (floppy disk image)
; It was created to work with IDE64 controller http://www.ide64.org
; 
;
; Copyright 1998-2001 Tomas Pribyl  (tomas.pribyl@ide64.org)
; Copyright 2004      Josef Soucek  (josef.soucek@ide64.org)
;
; This program is free Open Source software;
; Please feel free to use it.
; There is no warranty.
;-----------------------------------------------------------------------------

                   .include "kernal.asm"

buffer_lo = $fc
buffer_hi = $fd


quarter_buf = $1000

;version 1.0 - O.K.

;version 1.1 - Nov 03 1998 19:34:37 
;removed bug - now it works from all devices
;added       - skip all deleted files, include protected 

;version 1.2 - Apr 18 1999 12:04:52 
;rewritten for IDE64 plugin manager but unfinished

;version 1.3 - Nov 26 2001 15:47:00
;removed bug - wrong filename for writting

;version 1.4 - Dec 13 2001 19:41:51
;WORKING - separate directory for faster access

;version 1.5 - Nov 10 2004 21:18:00
;Detects and uses the seek (IDE-DOS >=V.9x)

;Const
source      = 2
destination = 3
command_ch  = 4

                   *= $0801

		   .byte $0b,$08,$e8,$03,$9e,$32,$30,$36,$31,$00,$00,$00
                   
                   lda #14
                   jsr chrout          ;lowercase
                   lda #8
                   jsr chrout          ;disable SHIFT/CBM
                   
                   lda #0
		   sta $9d             ;turn off messages

                   lda #<d64name
                   ldy #>d64name
                   jsr $ab1e           ;print utility name

		   ; Check the IDE64 availability

		   jsr check_ide64 
		   bcc ide64_detected
                   
                   lda #<noide64txt
                   ldy #>noide64txt
                   jsr $ab1e
		   
                   lda #0
		   sta IDE64		; For future CHRIN/CHROUT instead of READ/WRITE extension

		   rts			; No IDE64 - exit ###	
		   ;-------------
ide64_detected     = *
		   lda #1
		   sta IDE64 

                   ldy #0
getarc             jsr chrin
                   sta archivname,y
                   cmp #$0d
                   beq arcnameok
                   iny
                   cpy #$10            ;max.filename length
                   bne getarc

arcnameok          sty arclen          ;store archive filename lenght

                   jsr maketmpparts    ;creates 4 data parts and directory part
		   bcc open_ok_cont

                   lda #<d64opnerrtxt
                   ldy #>d64opnerrtxt
                   jsr $ab1e
		   rts		       ; Error Exit ###

open_ok_cont	   = *
                   jsr print_dir       ;reads and prints dir

                   lda #$00            ;searching in dir starts on first entry
                   sta entry
                   
                   lda #18             ;first DIR sector for searching
                   sta last_dir_trk
                   lda #1
                   sta last_dir_sect

                   lda #<input
                   ldy #>input
                   jsr $ab1e           ;print "input pattern"

                   ldy #0
getname            = *
		   jsr chrin
                   sta extractname,y   ;filename for extracting
                   cmp #$0d
                   beq end_name
                   iny
                   cpy #$10            ;max.filename length
                   bne getname
end_name           sty exnamelenth     ;store extract filenamelength

		   ; Extract loop...
                   
extract_next       jsr search_name     ;looking for file in directory
                   bcs end_extract     ;nothing found, end of extracting
                   
                   jsr extract_file    ;file found, extract it
                   jmp extract_next    ;repeat for all files

;------------------------------------------------------
; EXTRACTING FINISHED

end_extract        lda #source
                   jsr close
                   lda #destination
                   jsr close
		   lda #command_ch
		   jsr close
		   jsr clrchn
                   
                   lda SEEK
		   bne no_remove
                   jsr removetmpparts  ;delete temporary files
no_remove	   = *

                   lda #<end_txt       ;end of extracting
                   ldy #>end_txt
                   jmp $ab1e           ;END OF PROGRAM	###

;-------------------------------------------------------
; EXTRACT FILE FROM D64
; Input: X-sector beginning
;        Y-track beginning
;
; Ouptut : file on actual disk
;
;-------------------------------------------------------                   
extract_file       tya
                   pha
                   txa
                   pha                 ;store first sector pointer

		   jsr clrchn

                   ldy #$00
print_sign         lda extr_txt,y      ;print EXTRACTING
                   beq cont
                   jsr chrout
                   iny
                   jmp print_sign

cont               ldy #$00
tiskfile           lda ($bb),y
                   cmp #"/"
                   beq slash
                   cmp #"*"
                   bne notslash
slash              lda #"-"
                   sta ($bb),y
notslash           jsr chrout
                   iny
                   cpy $b7
                   bne tiskfile
                   
                   ldy $b7             ;add to filename ,x,W
                   lda #","
                   sta ($bb),y
                   iny
                   lda atribut
                   sta ($bb),y
                   iny
                   lda #","
                   sta ($bb),y
                   iny
                   lda #"W"
                   sta ($bb),y
                   iny
                   sty $b7
                   
                   inc $d3             ;move cursor
                   
                   
                   lda #destination
                   ldx $ba
                   ldy #1              ;open for writting
                   jsr setlfs
                   
                   jsr open
                   
                   pla                 ;restore first sector pointer
                   tax
                   pla
                   tay

;-----------------------------------------------------                   
cont_extract       jsr read_sector     ;read desired sector

                   ldx #destination
                   jsr chkout          ;set output to destination file
                   
                   lda #$36
                   sta $01             ;enable RAM
                   
                   ldy #$00
                   lda (buffer_lo),y
                   cmp #$00
                   beq lastsector
                   
;----------------------------------  store data
; store whole sector
                   ldy #$00
                   lda (buffer_lo),y
                   pha                 ;store next track
                   iny
                   lda (buffer_lo),y
                   pha                 ;store next sector
                   
                   clc
                   lda buffer_lo
                   adc #$02
                   sta buffer_lo       ;move pointer to first data byte in sector
                   
                   lda #buffer_lo
                   ldx #254            ;sector without first word (track,sect)
                   ldy #0
                   jsr hddwrite
                   
                   pla
                   tax                 ;restore next sector
                   pla
                   tay                 ;restore next track
                   jmp cont_extract    ;proceed next data


;---------------------------------------------------                   
; store rest of last sector
lastsector         ldy #$01
                   lda (buffer_lo),y
                   sta lenth_block
                   inc lenth_block     ;calculate offset of last byte
                   iny

store_last         lda (buffer_lo),y
                   jsr ChrOut
                   iny
                   cpy lenth_block
                   bne store_last
                   
                   ldy $d3
                   lda #122            ;put proceed char on screen
                   sta ($d1),y
                   
                   lda #destination
                   jsr close           ;close destination file
                   
                   lda #$37
                   sta $01             ;enable ROM
                   rts

;-----------------------------------------------------------------------------
; PRINTS DIR ON SCREEN

                   
print_dir          lda #$0d
                   jsr chrout          ;next line
                   
                   ldx #1              ;sector (begining of directory)
                   ldy #18             ;track
                   
next_dir_sect      jsr read_sector     ;reads 1. sector of directory

                   ldy #$00
                   lda (buffer_lo),y
                   sta next_trk        ;store number of next TRACK
                   iny
                   lda (buffer_lo),y
                   sta next_sect       ;store number of next SECTOR
                   
                   lda #$00
                   sta Entry           ;initialize entry in DIR to first
                   
print_entry        lda entry
                   asl
                   asl
                   asl
                   asl
                   asl
                   sta buffer_lo       ;calculate offset in actual sector
                   
                   ldy #$02
                   lda (buffer_lo),y   ;00-file deleted, next entry
                   beq next_entry
                   
                   and #$0f
                   tay
                   
                   lda type,y          ;type of file/dir
                   jsr chrout          ;put it on the screen
                   
                   lda #$20            ;wordpadding
                   jsr chrout
                   jsr chrout
                   

                   ldy #$05
printname          lda (buffer_lo),y   ;read filename from dir entry
                   bne printdirname
                   rts
                   
printdirname       jsr chrout
                   iny
                   cpy #$15            ;max.end of filename
                   bne printname
                   
                   lda #$20
                   jsr chrout          ;wordpadding
                   
                   ldy #$1e            ;blocks LOW
                   lda (buffer_lo),y
                   tax
                   iny
                   lda (buffer_lo),y   ;blocks HIGH
                   jsr $bdcd           ;print BLOCKS
                   
                   lda #$0d
                   jsr chrout          ;next line

;-------------------------------------------------------                   
next_entry         inc entry           ;next entry in dir
                   lda entry
                   cmp #$08
                   bne print_entry
                   
                   ldy next_trk        ;all entry in actual sector finished
                   cpy #$00
                   bne contnextdirsect ;test if it's last sector of dir
                   rts

contnextdirsect    ldx next_sect      
                   jmp next_dir_sect   ;no, continue (read next sector)

;------------------------------------------------------------------------------
;
; Looks for filename in directory and returns beginning of file
;
; Input: filename, filenamelength
; Output: track and sector of beginning file
;         if file was found then CY=0 otherwise CY=1
;------------------------------------------------------------------------------
search_name        lda #$36            ;RAM enable
                   sta $01
                   
                   ldx last_dir_sect
                   ldy last_dir_trk

next_dir_sect1     jsr read_sector     ;reads last readed dir sector

                   ldy #$00
                   lda (buffer_lo),y
                   sta next_trk        ;store next dir track
                   iny
                   lda (buffer_lo),y
                   sta next_sect       ;store next dir sector
                   
new_entry          lda entry
                   asl
                   asl
                   asl
                   asl
                   asl
                   sta buffer_lo

                   ldy #$02
                   lda (buffer_lo),y
                   and #%00111111      ;ignore VISIBLE & PROTECT bit
                   cmp #$00
                   beq next_search     ;DEL file skip too
                   
                   tay
                   lda type,y
                   sta atribut         ;store char of file attribut
                    
                   ldy #$05            ;beginning of filename in dir entry
                   ldx #$00            ;beginning of desired filename
compare_name       lda extractname,x
                   cmp #$2a            ;"*"
                   beq name_ok
                   cmp (buffer_lo),y
                   bne next_search     ;Different filename
                   iny
                   inx
                   cpx exnamelenth
                   bne compare_name 
                   
                   
name_ok            ldx #$00
                   ldy #$05
movename           lda (buffer_lo),y
                   cmp #$a0
                   beq endname
                   sta destname,x
                   iny
                   inx
                   cpx #$10            ;max.filenamelength
                   bne movename
                   
endname            txa                 ;filenamelength
                   ldx #<destname
                   ldy #>destname
                   jsr setnam          ;set original name for extracting
                   
                   inc entry           ;set next entry for searching
                   
                   lda entry           ;if entry is in new sector...
                   cmp #$08
                   bne sectnotpassed

                   lda #$00            ;...then initialize new entry...
                   sta entry
                   lda next_sect       ;...and set next dir track & sector
                   sta last_dir_sect   ;as last dir sector
                   lda next_trk
                   sta last_dir_trk
                   jmp getfilevector   ;get pointer of first file sector
                   
                   
sectnotpassed      lda sector          ;store actual sector for next searching
                   sta last_dir_sect
                   lda track           ;store actual track for next searching
                   sta last_dir_trk

getfilevector      ldy #$03            ;first TRACK of file
                   lda (buffer_lo),y
                   pha
                   iny                 ;first SECTOR of file
                   lda (buffer_lo),y
                   tax                 ;start sector of file
                   pla
                   tay                 ;start track of file
                   
                   lda #$37
                   sta $01             ;enable ROM
                   clc                 ;file was found OK.
                   rts
                   
;----------------------------------------------------------------------------
; Different filename
                   
next_search        inc entry           ;proceed next entry
                   lda entry
                   cmp #$08            ;last entry in this sector?
                   beq old             ;yes
                   jmp new_entry
                   
old                ldy next_trk        ;test if this directory sector is last
                   cpy #$00
                   beq end_of_dir1     ;yes, end of extracting
                   
                   ldx next_sect
                   lda #$00
                   sta entry           ;no, set to begining of next track
                   jmp next_dir_sect1
                   
end_of_dir1        lda #$37
                   sta $01             ;enable ROM
                   sec                 ;file not found, end of extracting
                   rts

;------------------------------------------------------------------------------
; rutine READ SECTOR
; Input: X sektor
;        Y track
;
; Output: BUFFER_LO/HI pointer to sector in RAM

read_sector        stx sector
                   sty track
                   
                   dey
                   tya                  ;track 1 is the first
                   asl                  ;*2
                   tay
                   
                   lda tracks_start,y
                   sta block_lo
                   lda tracks_start+1,y
                   sta block_hi         ;store linear sector number of the track
                   
                   clc                  ;and adds number of sectors in this track
                   lda sector
                   adc block_lo
                   sta block_lo
                   bcc not_cnt
                   inc block_hi         ;block_lo/hi is linear sector number
                   
not_cnt            ldy #$00
                   
divide             sec
                   lda block_lo
                   sbc #$ab             ;divide linear sector number by number
                   tax                  ;of sectors oncetime stored in RAM
                   lda block_hi
                   sbc #$00
                   bcc enddivide
                   
                   sta block_hi
                   stx block_lo
                   iny
                   jmp divide           ;continue dividing
                   
enddivide          sty quarter          ;number of quarter in which is actual sector located

                   lda block_lo         ;actual quarter is in RAM from $1000
                   clc                  ;therefore i must add $1000 to block_hi/lo
                   adc #>quarter_buf
                   sta block_lo
		   sta buffer_hi
		   lda #0
		   sta buffer_lo
                   
read               lda quarter
                   cmp act_quarter      ;test if required part is in RAM
		   bne read_the_part

                   ;part is already in RAM
                   rts

;---------------------------------  required data part isn't in RAM, load it...                   
read_the_part	   = *
                   sta act_quarter      ;store number of acutal part

		   ldx SEEK
		   beq cant_seek

		   jsr seek_quarter     ; Input: A - quarter #
                   jsr read_quarter
		   jsr clrchn
                   rts
		  
;------------------
cant_seek	   = *
                   clc                   
                   adc #"0"
                   sta tempfilename+10  ;convert to ASCII

                   ldx #<tempfilename+2 ;temporary filename
                   ldy #>tempfilename+2
                   lda #d64name-tempfilename+2
                   jsr setnam
                   
                   lda #source
                   ldx $ba
                   ldy #0
                   jsr setlfs

                   jsr open            ;open actual part
                   bcs error
                   
		   jsr read_quarter

                   lda #source
                   jsr close
	           jsr clrchn
		   rts
;--------------------------------------
seek_quarter	   = *
		   pha
		   ldx #command_ch
		   jsr chkout
		   
                   lda #"P"
		   jsr chrout
		   lda #source		; Channel to seek
		   jsr chrout
                   
		   lda #$00
                   jsr chrout			
		   pla
		   asl
		   tay
		   lda quarter_tab+1,y	; High
		   sta second_p+1
		   lda quarter_tab,y	; Low
		   jsr chrout
second_p	   lda #0
                   jsr chrout			
		   lda #0
                   jsr chrout			

		   jsr clrchn

	           ldx #command_ch
		   jsr chkin

		   jsr chrin
		   cmp #"0"
		   bne seek_aborted
                   jsr chrin
		   cmp #"0"
		   bne seek_aborted
		   jsr clrchn
		   clc
		   rts
seek_aborted	   jsr clrchn
		   sec
		   rts
;--------------------------------------
read_quarter	   = *
                   ldx #source
                   jsr chkin
                   
                   lda #<quarter_buf   ;beginning of databuffer
                   sta $fa
                   lda #>quarter_buf
                   sta $fb
                   
                   ldx #$00
                   ldy #$ab            ;read 171 sectors into buffer
                   lda #$fa
                   jsr hddread
                   bcs error
		   rts
;--------------------------------------
error              lda #$37
                   sta $01             ;enable ROM
                   jsr clall
		   jsr clrchn
                   lda #<err_txt
                   ldy #>err_txt
                   jsr $ab1e
	           jmp ($a002)		; BASIC cold restart
;----------------------------------------------------------------------------
;      Opens source D64 file, use the seek (IDE-DOS V.9x) 
;      or create 4 parts with 171 sectors
;      (It will rewrite TMP file if exists!)

maketmpparts       = *
                   lda #0             
                   sta act_quarter     ;initialize number of parts
		   sta SEEK	       ;initialize SEEK availability
                   
                   ; Open the command channel

		   lda #0
		   jsr setnam
		   lda #command_ch
		   ldx $ba
		   ldy #15
		   jsr setlfs
		   jsr open
                   bcs error

		   ; Open D64 file

                   ldx #<archivname    ;D64 filename
                   ldy #>archivname
                   lda arclen          ;D64 filenamelength
                   jsr setnam
                   lda #source
                   ldx $ba
                   ldy #source
;                   ldy #0		; Seek doesn't work with data channel sec.addres #0
                   jsr setlfs
                   jsr open
                   bcs error

		   ; Test if the file is opened correctly

	           ldx #command_ch
		   jsr chkin

		   jsr chrin
		   cmp #"0"
		   bne open_d64_err
                   jsr chrin
		   cmp #"0"
		   bne open_d64_err
		   jsr clrchn

		   ; Test the seek

		   lda #1
		   jsr seek_quarter
		   bcs seek_not_available

		   ; Set the SEEK variable

		   lda #1
		   sta SEEK

                   lda #<seektxt
                   ldy #>seektxt
                   jsr $ab1e

		   ; Seek is available, source and command ch. remains open

		   clc
	           rts
;------------------
open_d64_err	   = *
                   jsr clall
		   jsr clrchn
		   sec
		   rts 

; Write four parts/files

seek_not_available = *
		   lda #command_ch
		   jsr close

                   lda #<createtxt
                   ldy #>createtxt
                   jsr $ab1e

;------------------                   
                   lda #$36
                   sta $01             ;enable whole RAM

createnextpart     ldx #source
                   jsr chkin
                   
                   lda #<quarter_buf   ;beginning of databuffer
                   sta $fa
                   lda #>quarter_buf
                   sta $fb
                   
                   ldx #$00
                   ldy #$ab            ;read 171 sectors into buffer
                   lda #$fa            ;beginning - low byte pointer
                   jsr hddread
;                   bcs error
                   
                   txa
                   pha
                   tya
                   pha                 ;store readed block length

                   lda act_quarter     ;part number
                   clc
                   adc #"0"
                   sta tempfilename+10 ;convert it to ASCII

                   ldx #<tempfilename  ;temporary filename
                   ldy #>tempfilename
                   lda #d64name-tempfilename ; temporary filenamelength
                   jsr setnam
                   
                   lda #destination    ;destination file
                   ldx $ba
                   ldy #1              ;secondary address=write
                   jsr setlfs                           
                   
                   jsr open            ;create new part
;                   bcs error
                   
                   ldx #destination
                   jsr chkout          ;set output to opened file
                   
                   pla
                   tay
                   pla
                   tax                 ;restore block length
                   
                   lda #$fa
                   jsr hddwrite        ;write buffer to opened file
                   bcc writtenok
                   jmp error
                   
writtenok          lda #destination
                   jsr close
                   
                   inc act_quarter     ;next part
                   lda act_quarter 
                   cmp #$04            ;last part created?
                   bne createnextpart
                   
                   lda #source
                   jsr close
		   jsr clrchn
                   
                   lda #$37            ;enable ROM
                   sta $01
                   lda #<oktxt
                   ldy #>oktxt
                   jsr $ab1e
	           clc
                   rts

;----------------------------------------------------------------------------
removetmpparts     lda #<deletetxt
                   ldy #>deletetxt
                   jsr $ab1e           ;"Deleting temporary files..."

                   lda #$00
                   sta act_quarter     ;initialize part to be removed
                   
                   lda #"S"
                   sta tempfilename    ;put scratch char into filename
                   
removenexttmppart  lda act_quarter     ;part number
                   clc
                   adc #"0"
                   sta tempfilename+10 ;convert it to ASCII

                   ldx #<tempfilename  ;temporary filename
                   ldy #>tempfilename
                   lda #d64name-tempfilename ; temporary filenamelength
                   jsr setnam
                   
                   lda #destination    ;destination file
                   ldx $ba
                   ldy #15              ;secondary address=command
                   jsr setlfs                           
                   
                   jsr open            ;delete TMP file
                   
                   lda #destination
                   jsr close           ;close deleted file
                   
                   inc act_quarter
                   lda act_quarter
                   cmp #$04
                   bne removenexttmppart
                   
                   lda #<oktxt
                   ldy #>oktxt
                   jsr $ab1e           ;"OK."
                   rts
;--------------------------------------
check_ide64	   = *
		   lda $de60
		   cmp #"I"
		   bne ide64_not_present
		   lda $de61
		   cmp #"D"
		   bne ide64_not_present
		   lda $de62
		   cmp #"E"
		   bne ide64_not_present
		   clc
		   rts
ide64_not_present  sec
		   rts
;--------------------------------------
;Internal variables

next_trk           .byte 0 
next_sect          .byte 0   ;next sector pointer


sector             .byte 0   ;desired sector pointer
track              .byte 0
entry              .byte 0   ;actual entry in sector

block_lo           .byte 0   
block_hi           .byte 0   ;linear sector number

quarter            .byte 0   ;desired data part
act_quarter        .byte 0   ;actual data part in memory

exnamelenth        .byte 0   ;filenamelength of extracted file

last_dir_sect      .byte 0   ;last sector of directory (continue searching there)
last_dir_trk       .byte 0   ;last track of directory

lenth_block        .byte 0   ;delka posledniho sektoru

arclen             .byte 0   ;archieve filenamelength

atribut            .byte 0   ;file attribut

SEEK               .byte 0   ;IDE-DOS version >=9x supports file seeking
IDE64		   .byte 0   ;IDE-DOS presence
;---------------------------------------

type               .text "DSPUR"                ;attributes

archivname         .text "0123456789ABCDEF"     ;D64 filename

extractname        .text "0123456789ABCDEF"     ;extract filename

destname           .text "0123456789ABCDEF,p,r" ;extracted filename

TempFilename       .text "@:!D64TEMP0"          ;temporary filename


d64name            .text $93,"  read64 v1.5 by tOMAS pRIBYL (c) 2004",$0d
                   .text $0d,"d64 FILENAME: ",0

input              .text $0d,"nOTE: tYPE * TO EXTRACT ALL FILES",$0d
		   .text $0d,"INPUT PATTERN: ",0

extr_txt           .text $0d,"EXTRACTING: ",0

end_txt            .text $0d,$0d,"END OF EXTRACTING...",$0d,0

err_txt            .text $0d,"disk error!!!",0

createtxt          .text $0d,"cREATING TEMPORARY FILES... ",$0d,0
deletetxt          .text $0d,$0d,"dELETING TEMPORARY FILES... ",0
oktxt              .text "OK.",0
seektxt		   .text $0d,"sEEK AVAILABILITY DETECTED...",$0d,0

noide64txt	   .text $0d,"ide64 CARD NOT DETECTED!",$0d
		   .text "pLEASE VISIT HTTP://IDE64.ORG",$0d,0
d64opnerrtxt       .text $0d,"cAN'T OPEN d64 FILE...",$0d,0
;----------------------------------------------------------------------
; There are 171 sectors in the RAM buffer
; Contains linear sector #

tracks_start       .byte 0,0     ;track 1       21 sectors per track
                   .byte 21,0    ;track 2
                   .byte 42,0    ;track 3
                   .byte 63,0    ;track 4
                   .byte 84,0    ;track 5
                   .byte 105,0   ;track 6
                   .byte 126,0   ;track 7
                   .byte 147,0   ;track 8
                   .byte 168,0   ;track 9
                   .byte 189,0   ;track 10
                   .byte 210,0   ;track 11
                   .byte 231,0   ;track 12
                   .byte 252,0   ;track 13                      
                   .byte 17,1    ;track 14    (273)
                   .byte 38,1    ;track 15
                   .byte 59,1    ;track 16
                   .byte 80,1    ;track 17
                   .byte 101,1   ;track 18     19 sectors per track
                   .byte 120,1   ;track 19
                   .byte 139,1   ;track 20
                   .byte 158,1   ;track 21
                   .byte 177,1   ;track 22
                   .byte 196,1   ;track 23
                   .byte 215,1   ;track 24
                   .byte 234,1   ;track 25     18 sectors per track
                   .byte 252,1   ;track 26
                   .byte 14,2    ;track 27
                   .byte 32,2    ;track 28
                   .byte 50,2    ;track 29
                   .byte 68,2    ;track 30     (580)
                   .byte 86,2    ;track 31     17 sectors per track
                   .byte 103,2   ;track 32
                   .byte 120,2   ;track 33
                   .byte 137,2   ;track 34
                   .byte 154,2   ;track 35     (666)
;--------------------------------------
; Linear offset for each quarter buffer
;                        low,hi
quarter_tab	   .byte $00,$00
		   .byte $ab,$00
		   .byte $56,$01
		   .byte $01,$02
;--------------------------------------

